(function() {
  var PolarClock;

  this.PolarClock = PolarClock = (function() {

    function PolarClock(canvas, radius, opts) {
      var key, value;
      this.canvas = canvas;
      this.radius = radius;
      this.settings = {
        lineWidth: this.radius / 12,
        lineSpacing: 1,
        lineCap: 'butt',
        interval: 75,
        twelveHour: false,
        twoPhase: false
      };
      for (key in opts) {
        value = opts[key];
        this.settings[key] = value;
      }
    }

    PolarClock.prototype.start = function() {
      this.update();
      if (!this.updateIntervalID) {
        return this.updateIntervalID = setInterval(this.update.bind(this), this.settings.interval);
      }
    };

    PolarClock.prototype.stop = function() {
      if (this.updateIntervalID) {
        clearInterval(this.updateIntervalID);
        return this.updateIntervalID = null;
      }
    };

    PolarClock.prototype.update = function() {
      var day, daysInMonth, hour, min, month, msec, now, sec, weekday, yr;
      this.canvas.clearRect(0, 0, 2 * this.radius, 2 * this.radius);
      this.canvas.save();
      this.canvas.translate(this.radius, this.radius);
      this.canvas.scale(2, 2);
      this.canvas.rotate(-Math.PI / 2);
      this.canvas.lineWidth = this.settings.lineWidth - this.settings.lineSpacing;
      this.canvas.lineCap = this.settings.lineCap;
      now = new Date();
      daysInMonth = (function() {
        switch (now.getMonth()) {
          case 1:
            yr = now.getFullYear();
            if (yr % 400 === 0 || yr % 100 !== 0 && yr % 4 === 0) {
              return 29;
            } else {
              return 28;
            }
            break;
          case 3:
          case 5:
          case 8:
          case 10:
            return 30;
          default:
            return 31;
        }
      })();
      msec = now.getMilliseconds();
      sec = now.getSeconds() + msec / 1000;
      min = now.getMinutes() + sec / 60;
      hour = now.getHours() + min / 60;
      weekday = now.getDay() + hour / 24;
      day = now.getDate() - 1 + hour / 24;
      month = now.getMonth() + day / daysInMonth;
      if (this.settings.twoPhase) {
        this._writeTime(0, (sec % 30) / 30, sec > 30);
        this._writeTime(1, (min % 30) / 30, min > 30);
        if (this.settings.twelveHour) {
          this._writeTime(2, (hour % 6) / 6, (hour > 6 && hour < 12) || hour > 18);
        } else {
          this._writeTime(2, (hour % 12) / 12, hour > 12);
        }
        this._writeTime(3, ((weekday * 2) % 7) / 7, 2 * weekday > 7);
        this._writeTime(4, ((day * 2) % daysInMonth) / daysInMonth, 2 * day > daysInMonth);
        this._writeTime(5, (month % 6) / 6, month > 6);
      } else {
        this._writeTime(0, sec / 60);
        this._writeTime(1, min / 60);
        if (this.settings.twelveHour) {
          this._writeTime(2, (hour % 12) / 12, hour > 12);
        } else {
          this._writeTime(2, hour / 24);
        }
        this._writeTime(3, weekday / 7);
        this._writeTime(4, day / daysInMonth);
        this._writeTime(5, month / 12);
      }
      return this.canvas.restore();
    };

    PolarClock.prototype._writeTime = function(index, percent, waning) {
      if (waning == null) waning = false;
      this.canvas.save();
      this.canvas.strokeStyle = this._calculateColor(waning ? 1 - percent : percent);
      this.canvas.beginPath();
      this.canvas.arc(0, 0, (this.radius - (1 + 2 * index) * this.settings.lineWidth) / 2, 0, percent * 2 * Math.PI, waning);
      this.canvas.stroke();
      return this.canvas.restore();
    };

    PolarClock.prototype._calculateColor = function(percent) {
      var b, brightness, g;
      brightness = 255;
      b = percent * 255;
      g = brightness - b;
      return "rgba(0, " + (Math.round(g)) + ", " + (Math.round(b)) + ", 1)";
    };

    return PolarClock;

  })();

}).call(this);
